@@ -13,21 +13,30 @@
 #
 # Copyright Buildbot Team Members
 
-import mock
-import os.path
+
+import os
 import weakref
 
+import mock
+
+from twisted.internet import defer
+from twisted.internet import reactor
+from zope.interface import implementer
+
 from buildbot import config
 from buildbot import interfaces
 from buildbot.status import build
+from buildbot.test.fake import bworkermanager
+from buildbot.test.fake import fakedata
 from buildbot.test.fake import fakedb
+from buildbot.test.fake import fakemq
 from buildbot.test.fake import pbmanager
 from buildbot.test.fake.botmaster import FakeBotMaster
-from twisted.internet import defer
-from zope.interface import implements
+from buildbot.test.fake.machine import FakeMachineManager
+from buildbot.util import service
 
 
-class FakeCache(object):
+class FakeCache:
 
     """Emulate an L{AsyncLRUCache}, but without any real caching.  This
     I{does} do the weakref part, to catch un-weakref-able objects."""
@@ -39,53 +48,75 @@ class FakeCache(object):
     def get(self, key, **kwargs):
         d = self.miss_fn(key, **kwargs)
 
+        @d.addCallback
         def mkref(x):
             if x is not None:
                 weakref.ref(x)
             return x
-        d.addCallback(mkref)
         return d
 
     def put(self, key, val):
         pass
 
 
-class FakeCaches(object):
+class FakeCaches:
 
     def get_cache(self, name, miss_fn):
         return FakeCache(name, miss_fn)
 
 
-class FakeStatus(object):
+class FakeStatus(service.BuildbotService):
 
-    def __init__(self, master):
-        self.master = master
-        self.lastBuilderStatus = None
+    name = "status"
+    lastBuilderStatus = None
 
     def builderAdded(self, name, basedir, tags=None, description=None):
         bs = FakeBuilderStatus(self.master)
         self.lastBuilderStatus = bs
         return bs
 
-    def slaveConnected(self, name):
+    def getBuilderNames(self):
+        return []
+
+    def getWorkerNames(self):
+        return []
+
+    def workerConnected(self, name):
         pass
 
     def build_started(self, brid, buildername, build_status):
         pass
 
+    def getURLForBuild(self, builder_name, build_number):
+        return "URLForBuild/%s/%d" % (builder_name, build_number)
 
-class FakeBuilderStatus(object):
+    def getURLForBuildrequest(self, buildrequestid):
+        return "URLForBuildrequest/%d" % (buildrequestid,)
 
-    implements(interfaces.IBuilderStatus)
+    def subscribe(self, _):
+        pass
+
+    def getTitle(self):
+        return "myBuildbot"
+
+    def getURLForThing(self, _):
+        return "h://thing"
+
+    def getBuildbotURL(self):
+        return "h://bb.me"
+
+
+@implementer(interfaces.IBuilderStatus)
+class FakeBuilderStatus:
 
     def __init__(self, master=None, buildername="Builder"):
         if master:
             self.master = master
+            self.botmaster = master.botmaster
             self.basedir = os.path.join(master.basedir, 'bldr')
         self.lastBuildStatus = None
         self._tags = None
         self.name = buildername
-        self.status = mock.Mock()
 
     def setDescription(self, description):
         self._description = description
@@ -93,16 +124,16 @@ class FakeBuilderStatus(object):
     def getDescription(self):
         return self._description
 
-    def setTags(self, tags):
-        self._tags = tags
-
     def getTags(self):
         return self._tags
 
+    def setTags(self, tags):
+        self._tags = tags
+
     def matchesAnyTag(self, tags):
         return set(self._tags) & set(tags)
 
-    def setSlavenames(self, names):
+    def setWorkernames(self, names):
         pass
 
     def setCacheSize(self, size):
@@ -119,11 +150,13 @@ class FakeBuilderStatus(object):
     def buildStarted(self, builderStatus):
         pass
 
-    def addPointEvent(self, text):
-        pass
+
+class FakeLogRotation:
+    rotateLength = 42
+    maxRotatedFiles = 42
 
 
-class FakeMaster(object):
+class FakeMaster(service.MasterService):
 
     """
     Create a fake Master instance: a Mock with some convenience
@@ -132,16 +165,40 @@ class FakeMaster(object):
     - Non-caching implementation for C{self.caches}
     """
 
-    def __init__(self, master_id=fakedb.FakeBuildRequestsComponent.MASTER_ID):
+    def __init__(self, reactor,
+                 master_id=fakedb.FakeBuildRequestsComponent.MASTER_ID):
+        super().__init__()
         self._master_id = master_id
+        self.reactor = reactor
+        self.objectids = {}
         self.config = config.MasterConfig()
         self.caches = FakeCaches()
         self.pbmanager = pbmanager.FakePBManager()
+        self.initLock = defer.DeferredLock()
         self.basedir = 'basedir'
-        self.botmaster = FakeBotMaster(master=self)
-        self.botmaster.parent = self
-        self.status = FakeStatus(self)
-        self.status.master = self
+        self.botmaster = FakeBotMaster()
+        self.botmaster.setServiceParent(self)
+        self.status = FakeStatus()
+        self.status.setServiceParent(self)
+        self.name = 'fake:/master'
+        self.masterid = master_id
+        self.workers = bworkermanager.FakeWorkerManager()
+        self.workers.setServiceParent(self)
+        self.machine_manager = FakeMachineManager()
+        self.machine_manager.setServiceParent(self)
+        self.log_rotation = FakeLogRotation()
+        self.db = mock.Mock()
+        self.next_objectid = 0
+
+        def getObjectId(sched_name, class_name):
+            k = (sched_name, class_name)
+            try:
+                rv = self.objectids[k]
+            except KeyError:
+                rv = self.objectids[k] = self.next_objectid
+                self.next_objectid += 1
+            return defer.succeed(rv)
+        self.db.state.getObjectId = getObjectId
 
     def getObjectId(self):
         return defer.succeed(self._master_id)
@@ -149,19 +206,31 @@ class FakeMaster(object):
     def subscribeToBuildRequests(self, callback):
         pass
 
-    def maybeBuildsetComplete(self, bsid):
-        pass
-
-    # work around http://code.google.com/p/mock/issues/detail?id=105
-    def _get_child_mock(self, **kw):
-        return mock.Mock(**kw)
-
 # Leave this alias, in case we want to add more behavior later
 
 
-def make_master(wantDb=False, testcase=None, **kwargs):
-    master = FakeMaster(**kwargs)
+def make_master(testcase, wantMq=False, wantDb=False, wantData=False,
+                wantRealReactor=False, url=None, **kwargs):
+    if wantRealReactor:
+        _reactor = reactor
+    else:
+        assert testcase is not None, "need testcase for fake reactor"
+        # The test case must inherit from TestReactorMixin and setup it.
+        _reactor = testcase.reactor
+
+    master = FakeMaster(_reactor, **kwargs)
+    if url:
+        master.buildbotURL = url
+    if wantData:
+        wantMq = wantDb = True
+    if wantMq:
+        assert testcase is not None, "need testcase for wantMq"
+        master.mq = fakemq.FakeMQConnector(testcase)
+        master.mq.setServiceParent(master)
     if wantDb:
         assert testcase is not None, "need testcase for wantDb"
         master.db = fakedb.FakeDBConnector(testcase)
+        master.db.setServiceParent(master)
+    if wantData:
+        master.data = fakedata.FakeDataConnector(master, testcase)
     return master
