@@ -13,25 +13,43 @@
 #
 # Copyright Buildbot Team Members
 
-from __future__ import with_statement
-
-import mock
+import json
 import os
 import shutil
-import cStringIO as StringIO
 import sys
 
+import mock
+
 import twisted
 from twisted.internet import defer
-from twisted.trial import unittest
 from twisted.protocols import basic
+from twisted.python.compat import NativeStringIO
+from twisted.trial import unittest
+
 from buildbot.schedulers import trysched
 from buildbot.test.util import dirs
 from buildbot.test.util import scheduler
-from buildbot.util import json
+from buildbot.test.util.misc import TestReactorMixin
 
 
-class TryBase(unittest.TestCase):
+class TryBase(scheduler.SchedulerMixin, TestReactorMixin, unittest.TestCase):
+    OBJECTID = 26
+    SCHEDULERID = 6
+
+    def setUp(self):
+        self.setUpTestReactor()
+        self.setUpScheduler()
+
+    def tearDown(self):
+        self.tearDownScheduler()
+
+    def makeScheduler(self, **kwargs):
+        sched = self.attachScheduler(trysched.Try_Userpass(**kwargs),
+                                     self.OBJECTID, self.SCHEDULERID)
+        # Try will return a remote version of master.status, so give it
+        # something to return
+        sched.master.status = mock.Mock()
+        return sched
 
     def test_filterBuilderList_ok(self):
         sched = trysched.TryBase(
@@ -48,6 +66,35 @@ class TryBase(unittest.TestCase):
             name='tsched', builderNames=['a', 'b'], properties={})
         self.assertEqual(sched.filterBuilderList([]), ['a', 'b'])
 
+    @defer.inlineCallbacks
+    def test_enabled_callback(self):
+        sched = self.makeScheduler(name='tsched', builderNames=['a'],
+                                   port='tcp:9999', userpass=[('fred', 'derf')])
+        expectedValue = not sched.enabled
+        yield sched._enabledCallback(None, {'enabled': not sched.enabled})
+        self.assertEqual(sched.enabled, expectedValue)
+        expectedValue = not sched.enabled
+        yield sched._enabledCallback(None, {'enabled': not sched.enabled})
+        self.assertEqual(sched.enabled, expectedValue)
+
+    @defer.inlineCallbacks
+    def test_disabled_activate(self):
+        sched = self.makeScheduler(name='tsched', builderNames=['a'],
+                                   port='tcp:9999', userpass=[('fred', 'derf')])
+        yield sched._enabledCallback(None, {'enabled': not sched.enabled})
+        self.assertEqual(sched.enabled, False)
+        r = yield sched.activate()
+        self.assertEqual(r, None)
+
+    @defer.inlineCallbacks
+    def test_disabled_deactivate(self):
+        sched = self.makeScheduler(name='tsched', builderNames=['a'],
+                                   port='tcp:9999', userpass=[('fred', 'derf')])
+        yield sched._enabledCallback(None, {'enabled': not sched.enabled})
+        self.assertEqual(sched.enabled, False)
+        r = yield sched.deactivate()
+        self.assertEqual(r, None)
+
 
 class JobdirService(dirs.DirsMixin, unittest.TestCase):
 
@@ -62,30 +109,33 @@ class JobdirService(dirs.DirsMixin, unittest.TestCase):
         self.tearDownDirs()
 
     def test_messageReceived(self):
-        svc = trysched.JobdirService(self.jobdir)
+        # stub out svc.scheduler.handleJobFile and .jobdir
+        scheduler = mock.Mock()
 
-        # creat some new data to process
-        jobdata = os.path.join(self.newdir, 'jobdata')
-        with open(jobdata, "w") as f:
-            f.write('JOBDATA')
-
-        # stub out svc.parent.handleJobFile and .jobdir
         def handleJobFile(filename, f):
             self.assertEqual(filename, 'jobdata')
             self.assertEqual(f.read(), 'JOBDATA')
-        svc.parent = mock.Mock()
-        svc.parent.handleJobFile = handleJobFile
-        svc.parent.jobdir = self.jobdir
+        scheduler.handleJobFile = handleJobFile
+        scheduler.jobdir = self.jobdir
+
+        svc = trysched.JobdirService(scheduler=scheduler, basedir=self.jobdir)
+
+        # create some new data to process
+        jobdata = os.path.join(self.newdir, 'jobdata')
+        with open(jobdata, "w") as f:
+            f.write('JOBDATA')
 
         # run it
         svc.messageReceived('jobdata')
 
 
-class Try_Jobdir(scheduler.SchedulerMixin, unittest.TestCase):
+class Try_Jobdir(scheduler.SchedulerMixin, TestReactorMixin, unittest.TestCase):
 
     OBJECTID = 23
+    SCHEDULERID = 3
 
     def setUp(self):
+        self.setUpTestReactor()
         self.setUpScheduler()
         self.jobdir = None
 
@@ -96,7 +146,7 @@ class Try_Jobdir(scheduler.SchedulerMixin, unittest.TestCase):
 
     # tests
 
-    def do_test_startService(self, jobdir, exp_jobdir):
+    def setup_test_startService(self, jobdir, exp_jobdir):
         # set up jobdir
         self.jobdir = os.path.abspath('jobdir')
         if os.path.exists(self.jobdir):
@@ -106,65 +156,100 @@ class Try_Jobdir(scheduler.SchedulerMixin, unittest.TestCase):
         # build scheduler
         kwargs = dict(name="tsched", builderNames=['a'], jobdir=self.jobdir)
         sched = self.attachScheduler(
-            trysched.Try_Jobdir(**kwargs), self.OBJECTID)
+            trysched.Try_Jobdir(**kwargs), self.OBJECTID, self.SCHEDULERID,
+            overrideBuildsetMethods=True)
 
+        # watch interaction with the watcher service
+        sched.watcher.startService = mock.Mock()
+        sched.watcher.stopService = mock.Mock()
+
+    @defer.inlineCallbacks
+    def do_test_startService(self):
         # start it
-        sched.startService()
+        yield self.sched.startService()
 
         # check that it has set the basedir correctly
-        self.assertEqual(sched.watcher.basedir, self.jobdir)
+        self.assertEqual(self.sched.watcher.basedir, self.jobdir)
+        self.assertEqual(1, self.sched.watcher.startService.call_count)
+        self.assertEqual(0, self.sched.watcher.stopService.call_count)
 
-        return sched.stopService()
+        yield self.sched.stopService()
+
+        self.assertEqual(1, self.sched.watcher.startService.call_count)
+        self.assertEqual(1, self.sched.watcher.stopService.call_count)
 
     def test_startService_reldir(self):
-        return self.do_test_startService(
-                'jobdir',
-                os.path.abspath('basedir/jobdir'))
+        self.setup_test_startService(
+            'jobdir',
+            os.path.abspath('basedir/jobdir'))
+        return self.do_test_startService()
 
     def test_startService_reldir_subdir(self):
-        return self.do_test_startService(
-                'jobdir',
-                os.path.abspath('basedir/jobdir/cur'))
+        self.setup_test_startService(
+            'jobdir',
+            os.path.abspath('basedir/jobdir/cur'))
+        return self.do_test_startService()
 
     def test_startService_absdir(self):
-        return self.do_test_startService(
-                os.path.abspath('jobdir'),
-                os.path.abspath('jobdir'))
+        self.setup_test_startService(
+            os.path.abspath('jobdir'),
+            os.path.abspath('jobdir'))
+        return self.do_test_startService()
+
+    @defer.inlineCallbacks
+    def do_test_startService_but_not_active(self, jobdir, exp_jobdir):
+        """Same as do_test_startService, but the master wont activate this service"""
+        self.setup_test_startService(
+            'jobdir',
+            os.path.abspath('basedir/jobdir'))
+
+        self.setSchedulerToMaster(self.OTHER_MASTER_ID)
+
+        # start it
+        self.sched.startService()
+
+        # check that it has set the basedir correctly, even if it doesn't start
+        self.assertEqual(self.sched.watcher.basedir, self.jobdir)
+
+        yield self.sched.stopService()
+
+        self.assertEqual(0, self.sched.watcher.startService.call_count)
+        self.assertEqual(0, self.sched.watcher.stopService.call_count)
 
     # parseJob
 
     def test_parseJob_empty(self):
         sched = trysched.Try_Jobdir(
             name='tsched', builderNames=['a'], jobdir='foo')
-        self.assertRaises(
-            trysched.BadJobfile, sched.parseJob, StringIO.StringIO(''))
+        with self.assertRaises(trysched.BadJobfile):
+            sched.parseJob(NativeStringIO(''))
 
     def test_parseJob_longer_than_netstring_MAXLENGTH(self):
         self.patch(basic.NetstringReceiver, 'MAX_LENGTH', 100)
-        sched = trysched.Try_Jobdir(name='tsched', builderNames=['a'], jobdir='foo')
+        sched = trysched.Try_Jobdir(
+            name='tsched', builderNames=['a'], jobdir='foo')
         jobstr = self.makeNetstring(
             '1', 'extid', 'trunk', '1234', '1', 'this is my diff, -- ++, etc.',
             'buildera', 'builderc'
         )
         jobstr += 'x' * 200
 
-        test_temp_file = StringIO.StringIO(jobstr)
+        test_temp_file = NativeStringIO(jobstr)
 
-        self.assertRaises(trysched.BadJobfile,
-            lambda : sched.parseJob(test_temp_file))
+        with self.assertRaises(trysched.BadJobfile):
+            sched.parseJob(test_temp_file)
 
     def test_parseJob_invalid(self):
         sched = trysched.Try_Jobdir(
             name='tsched', builderNames=['a'], jobdir='foo')
-        self.assertRaises(
-            trysched.BadJobfile, sched.parseJob,
-            StringIO.StringIO('this is not a netstring'))
+        with self.assertRaises(trysched.BadJobfile):
+            sched.parseJob(NativeStringIO('this is not a netstring'))
 
     def test_parseJob_invalid_version(self):
         sched = trysched.Try_Jobdir(
             name='tsched', builderNames=['a'], jobdir='foo')
-        self.assertRaises(
-            trysched.BadJobfile, sched.parseJob, StringIO.StringIO('1:9,'))
+        with self.assertRaises(trysched.BadJobfile):
+            sched.parseJob(NativeStringIO('1:9,'))
 
     def makeNetstring(self, *strings):
         return ''.join(['%d:%s,' % (len(s), s) for s in strings])
@@ -176,7 +261,7 @@ class Try_Jobdir(scheduler.SchedulerMixin, unittest.TestCase):
             '1', 'extid', 'trunk', '1234', '1', 'this is my diff, -- ++, etc.',
             'buildera', 'builderc'
         )
-        parsedjob = sched.parseJob(StringIO.StringIO(jobstr))
+        parsedjob = sched.parseJob(NativeStringIO(jobstr))
         self.assertEqual(parsedjob, {
             'baserev': '1234',
             'branch': 'trunk',
@@ -195,11 +280,11 @@ class Try_Jobdir(scheduler.SchedulerMixin, unittest.TestCase):
         sched = trysched.Try_Jobdir(
             name='tsched', builderNames=['buildera', 'builderb'], jobdir='foo')
         jobstr = self.makeNetstring(
-                # blank branch, rev are turned to None
+            # blank branch, rev are turned to None
             '1', 'extid', '', '', '1', 'this is my diff, -- ++, etc.',
             'buildera', 'builderc'
         )
-        parsedjob = sched.parseJob(StringIO.StringIO(jobstr))
+        parsedjob = sched.parseJob(NativeStringIO(jobstr))
         self.assertEqual(parsedjob['branch'], None)
         self.assertEqual(parsedjob['baserev'], None)
 
@@ -209,7 +294,7 @@ class Try_Jobdir(scheduler.SchedulerMixin, unittest.TestCase):
         jobstr = self.makeNetstring(
             '1', 'extid', '', '', '1', 'this is my diff, -- ++, etc.'
         )
-        parsedjob = sched.parseJob(StringIO.StringIO(jobstr))
+        parsedjob = sched.parseJob(NativeStringIO(jobstr))
         self.assertEqual(parsedjob['builderNames'], [])
 
     def test_parseJob_v1_no_properties(self):
@@ -218,7 +303,7 @@ class Try_Jobdir(scheduler.SchedulerMixin, unittest.TestCase):
         jobstr = self.makeNetstring(
             '1', 'extid', '', '', '1', 'this is my diff, -- ++, etc.'
         )
-        parsedjob = sched.parseJob(StringIO.StringIO(jobstr))
+        parsedjob = sched.parseJob(NativeStringIO(jobstr))
         self.assertEqual(parsedjob['properties'], {})
 
     def test_parseJob_v2(self):
@@ -229,7 +314,7 @@ class Try_Jobdir(scheduler.SchedulerMixin, unittest.TestCase):
             'repo', 'proj',
             'buildera', 'builderc'
         )
-        parsedjob = sched.parseJob(StringIO.StringIO(jobstr))
+        parsedjob = sched.parseJob(NativeStringIO(jobstr))
         self.assertEqual(parsedjob, {
             'baserev': '1234',
             'branch': 'trunk',
@@ -248,12 +333,12 @@ class Try_Jobdir(scheduler.SchedulerMixin, unittest.TestCase):
         sched = trysched.Try_Jobdir(
             name='tsched', builderNames=['buildera', 'builderb'], jobdir='foo')
         jobstr = self.makeNetstring(
-                # blank branch, rev are turned to None
+            # blank branch, rev are turned to None
             '2', 'extid', '', '', '1', 'this is my diff, -- ++, etc.',
             'repo', 'proj',
             'buildera', 'builderc'
         )
-        parsedjob = sched.parseJob(StringIO.StringIO(jobstr))
+        parsedjob = sched.parseJob(NativeStringIO(jobstr))
         self.assertEqual(parsedjob['branch'], None)
         self.assertEqual(parsedjob['baserev'], None)
 
@@ -264,7 +349,7 @@ class Try_Jobdir(scheduler.SchedulerMixin, unittest.TestCase):
             '2', 'extid', 'trunk', '1234', '1', 'this is my diff, -- ++, etc.',
             'repo', 'proj',
         )
-        parsedjob = sched.parseJob(StringIO.StringIO(jobstr))
+        parsedjob = sched.parseJob(NativeStringIO(jobstr))
         self.assertEqual(parsedjob['builderNames'], [])
 
     def test_parseJob_v2_no_properties(self):
@@ -274,7 +359,7 @@ class Try_Jobdir(scheduler.SchedulerMixin, unittest.TestCase):
             '2', 'extid', 'trunk', '1234', '1', 'this is my diff, -- ++, etc.',
             'repo', 'proj',
         )
-        parsedjob = sched.parseJob(StringIO.StringIO(jobstr))
+        parsedjob = sched.parseJob(NativeStringIO(jobstr))
         self.assertEqual(parsedjob['properties'], {})
 
     def test_parseJob_v3(self):
@@ -285,7 +370,7 @@ class Try_Jobdir(scheduler.SchedulerMixin, unittest.TestCase):
             'repo', 'proj', 'who',
             'buildera', 'builderc'
         )
-        parsedjob = sched.parseJob(StringIO.StringIO(jobstr))
+        parsedjob = sched.parseJob(NativeStringIO(jobstr))
         self.assertEqual(parsedjob, {
             'baserev': '1234',
             'branch': 'trunk',
@@ -304,12 +389,12 @@ class Try_Jobdir(scheduler.SchedulerMixin, unittest.TestCase):
         sched = trysched.Try_Jobdir(
             name='tsched', builderNames=['buildera', 'builderb'], jobdir='foo')
         jobstr = self.makeNetstring(
-                # blank branch, rev are turned to None
+            # blank branch, rev are turned to None
             '3', 'extid', '', '', '1', 'this is my diff, -- ++, etc.',
             'repo', 'proj', 'who',
             'buildera', 'builderc'
         )
-        parsedjob = sched.parseJob(StringIO.StringIO(jobstr))
+        parsedjob = sched.parseJob(NativeStringIO(jobstr))
         self.assertEqual(parsedjob['branch'], None)
         self.assertEqual(parsedjob['baserev'], None)
 
@@ -320,7 +405,7 @@ class Try_Jobdir(scheduler.SchedulerMixin, unittest.TestCase):
             '3', 'extid', 'trunk', '1234', '1', 'this is my diff, -- ++, etc.',
             'repo', 'proj', 'who'
         )
-        parsedjob = sched.parseJob(StringIO.StringIO(jobstr))
+        parsedjob = sched.parseJob(NativeStringIO(jobstr))
         self.assertEqual(parsedjob['builderNames'], [])
 
     def test_parseJob_v3_no_properties(self):
@@ -330,7 +415,7 @@ class Try_Jobdir(scheduler.SchedulerMixin, unittest.TestCase):
             '3', 'extid', 'trunk', '1234', '1', 'this is my diff, -- ++, etc.',
             'repo', 'proj', 'who'
         )
-        parsedjob = sched.parseJob(StringIO.StringIO(jobstr))
+        parsedjob = sched.parseJob(NativeStringIO(jobstr))
         self.assertEqual(parsedjob['properties'], {})
 
     def test_parseJob_v4(self):
@@ -341,7 +426,7 @@ class Try_Jobdir(scheduler.SchedulerMixin, unittest.TestCase):
             'repo', 'proj', 'who', 'comment',
             'buildera', 'builderc'
         )
-        parsedjob = sched.parseJob(StringIO.StringIO(jobstr))
+        parsedjob = sched.parseJob(NativeStringIO(jobstr))
         self.assertEqual(parsedjob, {
             'baserev': '1234',
             'branch': 'trunk',
@@ -360,12 +445,12 @@ class Try_Jobdir(scheduler.SchedulerMixin, unittest.TestCase):
         sched = trysched.Try_Jobdir(
             name='tsched', builderNames=['buildera', 'builderb'], jobdir='foo')
         jobstr = self.makeNetstring(
-                # blank branch, rev are turned to None
+            # blank branch, rev are turned to None
             '4', 'extid', '', '', '1', 'this is my diff, -- ++, etc.',
             'repo', 'proj', 'who', 'comment',
             'buildera', 'builderc'
         )
-        parsedjob = sched.parseJob(StringIO.StringIO(jobstr))
+        parsedjob = sched.parseJob(NativeStringIO(jobstr))
         self.assertEqual(parsedjob['branch'], None)
         self.assertEqual(parsedjob['baserev'], None)
 
@@ -376,7 +461,7 @@ class Try_Jobdir(scheduler.SchedulerMixin, unittest.TestCase):
             '4', 'extid', 'trunk', '1234', '1', 'this is my diff, -- ++, etc.',
             'repo', 'proj', 'who', 'comment'
         )
-        parsedjob = sched.parseJob(StringIO.StringIO(jobstr))
+        parsedjob = sched.parseJob(NativeStringIO(jobstr))
         self.assertEqual(parsedjob['builderNames'], [])
 
     def test_parseJob_v4_no_properties(self):
@@ -386,7 +471,7 @@ class Try_Jobdir(scheduler.SchedulerMixin, unittest.TestCase):
             '4', 'extid', 'trunk', '1234', '1', 'this is my diff, -- ++, etc.',
             'repo', 'proj', 'who', 'comment'
         )
-        parsedjob = sched.parseJob(StringIO.StringIO(jobstr))
+        parsedjob = sched.parseJob(NativeStringIO(jobstr))
         self.assertEqual(parsedjob['properties'], {})
 
     def test_parseJob_v5(self):
@@ -401,7 +486,7 @@ class Try_Jobdir(scheduler.SchedulerMixin, unittest.TestCase):
                 'comment': 'comment', 'builderNames': ['buildera', 'builderc'],
                 'properties': {'foo': 'bar'},
             }))
-        parsedjob = sched.parseJob(StringIO.StringIO(jobstr))
+        parsedjob = sched.parseJob(NativeStringIO(jobstr))
         self.assertEqual(parsedjob, {
             'baserev': '1234',
             'branch': 'trunk',
@@ -420,12 +505,12 @@ class Try_Jobdir(scheduler.SchedulerMixin, unittest.TestCase):
         sched = trysched.Try_Jobdir(
             name='tsched', builderNames=['buildera', 'builderb'], jobdir='foo')
         jobstr = self.makeNetstring(
-                # blank branch, rev are turned to None
+            # blank branch, rev are turned to None
             '4', 'extid', '', '', '1', 'this is my diff, -- ++, etc.',
             'repo', 'proj', 'who', 'comment',
             'buildera', 'builderc'
         )
-        parsedjob = sched.parseJob(StringIO.StringIO(jobstr))
+        parsedjob = sched.parseJob(NativeStringIO(jobstr))
         self.assertEqual(parsedjob['branch'], None)
         self.assertEqual(parsedjob['baserev'], None)
 
@@ -441,7 +526,7 @@ class Try_Jobdir(scheduler.SchedulerMixin, unittest.TestCase):
                 'comment': 'comment', 'builderNames': [],
                 'properties': {'foo': 'bar'},
             }))
-        parsedjob = sched.parseJob(StringIO.StringIO(jobstr))
+        parsedjob = sched.parseJob(NativeStringIO(jobstr))
         self.assertEqual(parsedjob['builderNames'], [])
 
     def test_parseJob_v5_no_properties(self):
@@ -456,15 +541,15 @@ class Try_Jobdir(scheduler.SchedulerMixin, unittest.TestCase):
                 'comment': 'comment', 'builderNames': ['buildera', 'builderb'],
                 'properties': {},
             }))
-        parsedjob = sched.parseJob(StringIO.StringIO(jobstr))
+        parsedjob = sched.parseJob(NativeStringIO(jobstr))
         self.assertEqual(parsedjob['properties'], {})
 
     def test_parseJob_v5_invalid_json(self):
         sched = trysched.Try_Jobdir(
             name='tsched', builderNames=['buildera', 'builderb'], jobdir='foo')
         jobstr = self.makeNetstring('5', '{"comment": "com}')
-        self.assertRaises(
-            trysched.BadJobfile, sched.parseJob, StringIO.StringIO(jobstr))
+        with self.assertRaises(trysched.BadJobfile):
+            sched.parseJob(NativeStringIO(jobstr))
 
     # handleJobFile
 
@@ -472,116 +557,123 @@ class Try_Jobdir(scheduler.SchedulerMixin, unittest.TestCase):
         sched = self.attachScheduler(
             trysched.Try_Jobdir(
                 name='tsched', builderNames=['buildera', 'builderb'],
-                jobdir='foo'), self.OBJECTID)
+                jobdir='foo'), self.OBJECTID, self.SCHEDULERID,
+            overrideBuildsetMethods=True,
+            createBuilderDB=True)
         fakefile = mock.Mock()
 
         def parseJob_(f):
             assert f is fakefile
             return parseJob(f)
         sched.parseJob = parseJob_
-        return sched.handleJobFile('fakefile', fakefile)
+        return defer.maybeDeferred(sched.handleJobFile, 'fakefile', fakefile)
 
     def makeSampleParsedJob(self, **overrides):
         pj = dict(baserev='1234', branch='trunk',
-            builderNames=['buildera', 'builderb'],
-            jobid='extid', patch_body='this is my diff, -- ++, etc.',
-            patch_level=1, project='proj', repository='repo', who='who',
-            comment='comment', properties={})
+                  builderNames=['buildera', 'builderb'],
+                  jobid='extid', patch_body='this is my diff, -- ++, etc.',
+                  patch_level=1, project='proj', repository='repo', who='who',
+                  comment='comment', properties={})
         pj.update(overrides)
         return pj
 
+    @defer.inlineCallbacks
     def test_handleJobFile(self):
-        d = self.call_handleJobFile(lambda f: self.makeSampleParsedJob())
+        yield self.call_handleJobFile(lambda f: self.makeSampleParsedJob())
 
-        def check(_):
-            self.db.buildsets.assertBuildset('?',
-                    dict(reason="'try' job by user who",
-                        external_idstring='extid',
-                        properties=[('scheduler', ('tsched', 'Scheduler'))],
-                        sourcestampsetid=100),
-                    {'':
-                     dict(branch='trunk', repository='repo',  codebase='',
-                        project='proj', revision='1234',
-                        patch_body='this is my diff, -- ++, etc.',
-                        patch_level=1, patch_subdir='',
+        self.assertEqual(self.addBuildsetCalls, [
+            ('addBuildsetForSourceStamps', dict(
+                builderNames=['buildera', 'builderb'],
+                external_idstring='extid',
+                properties={},
+                reason="'try' job by user who",
+                sourcestamps=[
+                    dict(
+                        branch='trunk',
+                        codebase='',
                         patch_author='who',
+                        patch_body='this is my diff, -- ++, etc.',
                         patch_comment='comment',
-                        sourcestampsetid=100)
-                    })
-        d.addCallback(check)
-        return d
+                        patch_level=1,
+                        patch_subdir='',
+                        project='proj',
+                        repository='repo',
+                        revision='1234'),
+                ])),
+        ])
 
+    @defer.inlineCallbacks
     def test_handleJobFile_exception(self):
         def parseJob(f):
             raise trysched.BadJobfile
-        d = self.call_handleJobFile(parseJob)
+        yield self.call_handleJobFile(parseJob)
 
-        def check(bsid):
-            self.db.buildsets.assertBuildsets(0)
-            self.assertEqual(
-                1, len(self.flushLoggedErrors(trysched.BadJobfile)))
-        d.addCallback(check)
-        return d
+        self.assertEqual(self.addBuildsetCalls, [])
+        self.assertEqual(
+            1, len(self.flushLoggedErrors(trysched.BadJobfile)))
     if twisted.version.major <= 9 and sys.version_info[:2] >= (2, 7):
         test_handleJobFile_exception.skip = (
             "flushLoggedErrors does not work correctly on 9.0.0 "
             "and earlier with Python-2.7")
 
+    @defer.inlineCallbacks
     def test_handleJobFile_bad_builders(self):
-        d = self.call_handleJobFile(
-                lambda f: self.makeSampleParsedJob(builderNames=['xxx']))
+        yield self.call_handleJobFile(
+            lambda f: self.makeSampleParsedJob(builderNames=['xxx']))
 
-        def check(_):
-            self.db.buildsets.assertBuildsets(0)
-        d.addCallback(check)
-        return d
+        self.assertEqual(self.addBuildsetCalls, [])
 
+    @defer.inlineCallbacks
     def test_handleJobFile_subset_builders(self):
-        d = self.call_handleJobFile(
+        yield self.call_handleJobFile(
             lambda f: self.makeSampleParsedJob(builderNames=['buildera']))
 
-        def check(_):
-            self.db.buildsets.assertBuildset('?',
-                    dict(reason="'try' job by user who",
-                        external_idstring='extid',
-                        properties=[('scheduler', ('tsched', 'Scheduler'))],
-                        sourcestampsetid=100),
-                    {'':
-                     dict(branch='trunk', repository='repo',  codebase='',
-                        project='proj', revision='1234',
-                        patch_body='this is my diff, -- ++, etc.',
-                        patch_level=1, patch_subdir='',
+        self.assertEqual(self.addBuildsetCalls, [
+            ('addBuildsetForSourceStamps', dict(
+                builderNames=['buildera'],
+                external_idstring='extid',
+                properties={},
+                reason="'try' job by user who",
+                sourcestamps=[
+                    dict(
+                        branch='trunk',
+                        codebase='',
                         patch_author='who',
+                        patch_body='this is my diff, -- ++, etc.',
                         patch_comment='comment',
-                        sourcestampsetid=100)
-                    })
-        d.addCallback(check)
-        return d
+                        patch_level=1,
+                        patch_subdir='',
+                        project='proj',
+                        repository='repo',
+                        revision='1234'),
+                ])),
+        ])
 
+    @defer.inlineCallbacks
     def test_handleJobFile_with_try_properties(self):
-        d = self.call_handleJobFile(
-            lambda f: self.makeSampleParsedJob(properties={'foo': 'bar'}))
+        yield self.call_handleJobFile(
+                lambda f: self.makeSampleParsedJob(properties={'foo': 'bar'}))
 
-        def check(_):
-            self.db.buildsets.assertBuildset('?',
-                    dict(reason="'try' job by user who",
-                        external_idstring='extid',
-                        properties=[
-                            ('foo', ('bar', 'try build')),
-                            ('scheduler', ('tsched', 'Scheduler')),
-                        ],
-                        sourcestampsetid=100),
-                    {'':
-                     dict(branch='trunk', repository='repo',  codebase='',
-                        project='proj', revision='1234',
-                        patch_body='this is my diff, -- ++, etc.',
-                        patch_level=1, patch_subdir='',
+        self.assertEqual(self.addBuildsetCalls, [
+            ('addBuildsetForSourceStamps', dict(
+                builderNames=['buildera', 'builderb'],
+                external_idstring='extid',
+                properties={'foo': ('bar', 'try build')},
+                reason="'try' job by user who",
+                sourcestamps=[
+                    dict(
+                        branch='trunk',
+                        codebase='',
                         patch_author='who',
+                        patch_body='this is my diff, -- ++, etc.',
                         patch_comment='comment',
-                        sourcestampsetid=100)
-                    })
-        d.addCallback(check)
-        return d
+                        patch_level=1,
+                        patch_subdir='',
+                        project='proj',
+                        repository='repo',
+                        revision='1234'),
+                ])),
+        ])
 
     def test_handleJobFile_with_invalid_try_properties(self):
         d = self.call_handleJobFile(
@@ -589,11 +681,14 @@ class Try_Jobdir(scheduler.SchedulerMixin, unittest.TestCase):
         return self.assertFailure(d, AttributeError)
 
 
-class Try_Userpass_Perspective(scheduler.SchedulerMixin, unittest.TestCase):
+class Try_Userpass_Perspective(scheduler.SchedulerMixin, TestReactorMixin,
+                               unittest.TestCase):
 
     OBJECTID = 26
+    SCHEDULERID = 6
 
     def setUp(self):
+        self.setUpTestReactor()
         self.setUpScheduler()
 
     def tearDown(self):
@@ -601,98 +696,113 @@ class Try_Userpass_Perspective(scheduler.SchedulerMixin, unittest.TestCase):
 
     def makeScheduler(self, **kwargs):
         sched = self.attachScheduler(trysched.Try_Userpass(**kwargs),
-                self.OBJECTID)
+                                     self.OBJECTID, self.SCHEDULERID,
+                                     overrideBuildsetMethods=True,
+                                     createBuilderDB=True)
         # Try will return a remote version of master.status, so give it
         # something to return
         sched.master.status = mock.Mock()
         return sched
 
+    @defer.inlineCallbacks
     def call_perspective_try(self, *args, **kwargs):
         sched = self.makeScheduler(name='tsched', builderNames=['a', 'b'],
-                port='xxx', userpass=[('a', 'b')], properties=dict(frm='schd'))
+                                   port='xxx', userpass=[('a', 'b')],
+                                   properties=dict(frm='schd'))
         persp = trysched.Try_Userpass_Perspective(sched, 'a')
-        return persp.perspective_try(*args, **kwargs)
 
+        # patch out all of the handling after addBuildsetForSourceStamp
+        def getBuildset(bsid):
+            return dict(bsid=bsid)
+        self.db.buildsets.getBuildset = getBuildset
+
+        rbss = yield persp.perspective_try(*args, **kwargs)
+
+        if rbss is None:
+            return
+        self.assertIsInstance(rbss, trysched.RemoteBuildSetStatus)
+
+    @defer.inlineCallbacks
     def test_perspective_try(self):
-        d = self.call_perspective_try(
+        yield self.call_perspective_try(
             'default', 'abcdef', (1, '-- ++'), 'repo', 'proj', ['a'],
             properties={'pr': 'op'})
 
-        def check(_):
-            self.db.buildsets.assertBuildset('?',
-                    dict(reason="'try' job",
-                        external_idstring=None,
-                        properties=[
-                            ('frm', ('schd', 'Scheduler')),
-                            ('pr', ('op', 'try build')),
-                            ('scheduler', ('tsched', 'Scheduler')),
-                        ],
-                        sourcestampsetid=100,
-                        ),
-                    {'':
-                     dict(branch='default', repository='repo', codebase='',
-                        project='proj', revision='abcdef',
-                        sourcestampsetid=100,
-                        patch_body='-- ++', patch_level=1, patch_subdir='',
-                        patch_author="", patch_comment="")
-                    })
-        d.addCallback(check)
-        return d
+        self.assertEqual(self.addBuildsetCalls, [
+            ('addBuildsetForSourceStamps', dict(
+                builderNames=['a'],
+                external_idstring=None,
+                properties={'pr': ('op', 'try build')},
+                reason="'try' job",
+                sourcestamps=[
+                    dict(
+                        branch='default',
+                        codebase='',
+                        patch_author='',
+                        patch_body='-- ++',
+                        patch_comment='',
+                        patch_level=1,
+                        patch_subdir='',
+                        project='proj',
+                        repository='repo',
+                        revision='abcdef'),
+                ])),
+        ])
 
+    @defer.inlineCallbacks
     def test_perspective_try_who(self):
-        d = self.call_perspective_try(
-            'default', 'abcdef', (1, '-- ++'), 'repo', 'proj', ['a'],
-            who='who', comment='comment', properties={'pr': 'op'})
+        yield self.call_perspective_try(
+                'default', 'abcdef', (1, '-- ++'), 'repo', 'proj', ['a'],
+                who='who', comment='comment', properties={'pr': 'op'})
 
-        def check(_):
-            self.db.buildsets.assertBuildset('?',
-                    dict(reason="'try' job by user who (comment)",
-                        external_idstring=None,
-                        properties=[
-                            ('frm', ('schd', 'Scheduler')),
-                            ('pr', ('op', 'try build')),
-                            ('scheduler', ('tsched', 'Scheduler')),
-                        ],
-                        sourcestampsetid=100,
-                        ),
-                    {'':
-                     dict(branch='default', repository='repo',  codebase='',
-                        project='proj', revision='abcdef',
-                        sourcestampsetid=100,
-                        patch_body='-- ++', patch_level=1, patch_subdir='',
-                        patch_author='who', patch_comment="comment")
-                    })
-        d.addCallback(check)
-        return d
+        self.assertEqual(self.addBuildsetCalls, [
+            ('addBuildsetForSourceStamps', dict(
+                builderNames=['a'],
+                external_idstring=None,
+                properties={'pr': ('op', 'try build')},
+                reason="'try' job by user who (comment)",
+                sourcestamps=[
+                    dict(
+                        branch='default',
+                        codebase='',
+                        patch_author='who',
+                        patch_body='-- ++',
+                        patch_comment='comment',
+                        patch_level=1,
+                        patch_subdir='',
+                        project='proj',
+                        repository='repo',
+                        revision='abcdef'),
+                ])),
+        ])
 
+    @defer.inlineCallbacks
     def test_perspective_try_bad_builders(self):
-        d = self.call_perspective_try(
-            'default', 'abcdef', (1, '-- ++'), 'repo', 'proj', ['xxx'],
-            properties={'pr': 'op'})
+        yield self.call_perspective_try(
+                'default', 'abcdef', (1, '-- ++'), 'repo', 'proj', ['xxx'],
+                properties={'pr': 'op'})
 
-        def check(_):
-            self.db.buildsets.assertBuildsets(0)
-        d.addCallback(check)
-        return d
+        self.assertEqual(self.addBuildsetCalls, [])
 
+    @defer.inlineCallbacks
     def test_getAvailableBuilderNames(self):
         sched = self.makeScheduler(name='tsched', builderNames=['a', 'b'],
-                port='xxx', userpass=[('a', 'b')])
+                                   port='xxx', userpass=[('a', 'b')])
         persp = trysched.Try_Userpass_Perspective(sched, 'a')
-        d = defer.maybeDeferred(
-            lambda: persp.perspective_getAvailableBuilderNames())
+        buildernames = yield defer.maybeDeferred(
+                                persp.perspective_getAvailableBuilderNames)
 
-        def check(buildernames):
-            self.assertEqual(buildernames, ['a', 'b'])
-        d.addCallback(check)
-        return d
+        self.assertEqual(buildernames, ['a', 'b'])
 
 
-class Try_Userpass(scheduler.SchedulerMixin, unittest.TestCase):
+class Try_Userpass(scheduler.SchedulerMixin, TestReactorMixin,
+                   unittest.TestCase):
 
     OBJECTID = 25
+    SCHEDULERID = 5
 
     def setUp(self):
+        self.setUpTestReactor()
         self.setUpScheduler()
 
     def tearDown(self):
@@ -700,12 +810,12 @@ class Try_Userpass(scheduler.SchedulerMixin, unittest.TestCase):
 
     def makeScheduler(self, **kwargs):
         sched = self.attachScheduler(trysched.Try_Userpass(**kwargs),
-                self.OBJECTID)
+                                     self.OBJECTID, self.SCHEDULERID)
         return sched
 
     def test_service(self):
         sched = self.makeScheduler(name='tsched', builderNames=['a'],
-                port='tcp:9999', userpass=[('fred', 'derf')])
+                                   port='tcp:9999', userpass=[('fred', 'derf')])
         # patch out the pbmanager's 'register' command both to be sure
         # the registration is correct and to get a copy of the factory
         registration = mock.Mock()
@@ -723,5 +833,19 @@ class Try_Userpass(scheduler.SchedulerMixin, unittest.TestCase):
         # make a fake connection by invoking the factory, and check that we
         # get the correct perspective
         persp = self.got_factory(mock.Mock(), 'fred')
-        self.failUnless(isinstance(persp, trysched.Try_Userpass_Perspective))
+        self.assertTrue(isinstance(persp, trysched.Try_Userpass_Perspective))
         return sched.stopService()
+
+    @defer.inlineCallbacks
+    def test_service_but_not_active(self):
+        sched = self.makeScheduler(name='tsched', builderNames=['a'],
+                                   port='tcp:9999', userpass=[('fred', 'derf')])
+
+        self.setSchedulerToMaster(self.OTHER_MASTER_ID)
+
+        sched.master.pbmanager = mock.Mock()
+
+        sched.startService()
+        yield sched.stopService()
+
+        self.assertFalse(sched.master.pbmanager.register.called)
