@@ -123,7 +123,6 @@ class IEmailSender(Interface):
 
     """I know how to send email, and can be used by other parts of the
     Buildbot to contact developers."""
-    pass
 
 
 class IEmailLookup(Interface):
@@ -188,6 +187,15 @@ class ILatentWorker(IWorker):
         """
 
 
+class IMachine(Interface):
+    pass
+
+
+class ILatentMachine(IMachine):
+    """ A machine that is not always running, but can be started when requested.
+    """
+
+
 class IRenderable(Interface):
 
     """An object that can be interpolated with properties from a build.
