@@ -13,8 +13,6 @@
 #
 # Copyright Buildbot Team Members
 
-from future.utils import text_type
-
 from twisted.internet import defer
 from twisted.python import log
 
@@ -22,7 +20,7 @@ from buildbot import util
 from buildbot.util import lineboundaries
 
 
-class FakeLogFile(object):
+class FakeLogFile:
 
     def __init__(self, name, step):
         self.name = name
@@ -46,7 +44,7 @@ class FakeLogFile(object):
             return self.lbfs[stream]
         except KeyError:
             def wholeLines(lines):
-                if not isinstance(lines, text_type):
+                if not isinstance(lines, str):
                     lines = lines.decode('utf-8')
                 if self.name in self.step.logobservers:
                     for obs in self.step.logobservers[self.name]:
