@@ -17,7 +17,6 @@ import mock
 
 from twisted.internet import defer
 from twisted.internet import error
-from twisted.internet import reactor
 from twisted.python import failure
 from twisted.python.compat import NativeStringIO
 from twisted.trial import unittest
@@ -33,6 +32,7 @@ from buildbot.steps import shell
 from buildbot.test.fake import fakedb
 from buildbot.test.fake import fakemaster
 from buildbot.test.fake import fakeprotocol
+from buildbot.test.util.misc import TestReactorMixin
 from buildbot.worker.base import Worker
 
 
@@ -47,15 +47,16 @@ class TestLogObserver(buildstep.LogObserver):
 
 class OldStyleCustomBuildStep(buildstep.BuildStep):
 
-    def __init__(self, arg1, arg2, doFail=False, **kwargs):
-        buildstep.BuildStep.__init__(self, **kwargs)
+    def __init__(self, reactor, arg1, arg2, doFail=False, **kwargs):
+        super().__init__(**kwargs)
+        self.reactor = reactor
         self.arg1 = arg1
         self.arg2 = arg2
         self.doFail = doFail
 
     def start(self):
         # don't complete immediately, or synchronously
-        reactor.callLater(0, self.doStuff)
+        self.reactor.callLater(0, self.doStuff)
 
     def doStuff(self):
         try:
@@ -104,7 +105,7 @@ class FailingCustomStep(buildstep.LoggingBuildStep):
     flunkOnFailure = True
 
     def __init__(self, exception=buildstep.BuildStepFailed, *args, **kwargs):
-        buildstep.LoggingBuildStep.__init__(self, *args, **kwargs)
+        super().__init__(*args, **kwargs)
         self.exception = exception
 
     @defer.inlineCallbacks
@@ -151,12 +152,13 @@ class OldPerlModuleTest(shell.Test):
         return results.SUCCESS
 
 
-class RunSteps(unittest.TestCase):
+class RunSteps(unittest.TestCase, TestReactorMixin):
 
     @defer.inlineCallbacks
     def setUp(self):
-        self.master = fakemaster.make_master(testcase=self,
-                                             wantData=True, wantMq=True, wantDb=True)
+        self.setUpTestReactor()
+        self.master = fakemaster.make_master(self, wantData=True,
+                                             wantMq=True, wantDb=True)
         self.master.db.insertTestData([
             fakedb.Builder(id=80, name='test'), ])
 
@@ -239,7 +241,7 @@ class RunSteps(unittest.TestCase):
 
         def finishNewLog(self):
             for d in newLogDeferreds:
-                reactor.callLater(0, d.callback, None)
+                self.reactor.callLater(0, d.callback, None)
 
         def delayedNewLog(*args, **kwargs):
             d = defer.Deferred()
@@ -252,7 +254,8 @@ class RunSteps(unittest.TestCase):
             self.patch(OldStyleCustomBuildStep,
                        "_run_finished_hook", finishNewLog)
 
-        self.factory.addStep(OldStyleCustomBuildStep(arg1=1, arg2=2))
+        self.factory.addStep(OldStyleCustomBuildStep(self.reactor,
+                                                     arg1=1, arg2=2))
         yield self.do_test_step()
 
         self.assertLogs({
@@ -278,7 +281,8 @@ class RunSteps(unittest.TestCase):
 
     @defer.inlineCallbacks
     def test_OldStyleCustomBuildStep_failure(self):
-        self.factory.addStep(OldStyleCustomBuildStep(arg1=1, arg2=2, doFail=1))
+        self.factory.addStep(OldStyleCustomBuildStep(self.reactor,
+                                                     arg1=1, arg2=2, doFail=1))
         bs = yield self.do_test_step()
         self.assertEqual(len(self.flushLoggedErrors(RuntimeError)), 1)
         self.assertEqual(bs.getResults(), results.EXCEPTION)
