@@ -13,8 +13,6 @@
 #
 # Copyright Buildbot Team Members
 
-from future.utils import text_type
-
 import random
 
 import mock
@@ -30,16 +28,17 @@ from buildbot.process.properties import Properties
 from buildbot.process.properties import renderer
 from buildbot.test.fake import fakedb
 from buildbot.test.fake import fakemaster
+from buildbot.test.util.misc import TestReactorMixin
 from buildbot.test.util.warnings import assertProducesWarning
 from buildbot.util import epoch2datetime
 from buildbot.worker import AbstractLatentWorker
 
 
-class BuilderMixin(object):
+class BuilderMixin:
 
     def setUpBuilderMixin(self):
         self.factory = factory.BuildFactory()
-        self.master = fakemaster.make_master(testcase=self, wantData=True)
+        self.master = fakemaster.make_master(self, wantData=True)
         self.mq = self.master.mq
         self.db = self.master.db
 
@@ -79,16 +78,17 @@ class BuilderMixin(object):
             return self.bldr.reconfigServiceWithBuildbotConfig(mastercfg)
 
 
-class FakeWorker(object):
+class FakeWorker:
     builds_may_be_incompatible = False
 
     def __init__(self, workername):
         self.workername = workername
 
 
-class TestBuilder(BuilderMixin, unittest.TestCase):
+class TestBuilder(TestReactorMixin, BuilderMixin, unittest.TestCase):
 
     def setUp(self):
+        self.setUpTestReactor()
         # a collection of rows that would otherwise clutter up every test
         self.setUpBuilderMixin()
         self.base_rows = [
@@ -99,8 +99,7 @@ class TestBuilder(BuilderMixin, unittest.TestCase):
 
     @defer.inlineCallbacks
     def makeBuilder(self, patch_random=False, startBuildsForSucceeds=True, **config_kwargs):
-        yield BuilderMixin.makeBuilder(
-            self, patch_random=patch_random, **config_kwargs)
+        yield super().makeBuilder(patch_random=patch_random, **config_kwargs)
 
         # patch into the _startBuildsFor method
         self.builds_started = []
@@ -418,7 +417,7 @@ class TestBuilder(BuilderMixin, unittest.TestCase):
     @defer.inlineCallbacks
     def test_getBuilderId(self):
         self.factory = factory.BuildFactory()
-        self.master = fakemaster.make_master(testcase=self, wantData=True)
+        self.master = fakemaster.make_master(self, wantData=True)
         # only include the necessary required config, plus user-requested
         self.bldr = builder.Builder('bldr')
         self.bldr.master = self.master
@@ -458,9 +457,10 @@ class TestBuilder(BuilderMixin, unittest.TestCase):
         self.assertEquals(props.getProperty('cuckoo'), 42)
 
 
-class TestGetBuilderId(BuilderMixin, unittest.TestCase):
+class TestGetBuilderId(TestReactorMixin, BuilderMixin, unittest.TestCase):
 
     def setUp(self):
+        self.setUpTestReactor()
         self.setUpBuilderMixin()
 
     @defer.inlineCallbacks
@@ -477,13 +477,15 @@ class TestGetBuilderId(BuilderMixin, unittest.TestCase):
         fbi.assert_called_once_with('b1')
         # check that the name was unicodified
         arg = fbi.mock_calls[0][1][0]
-        self.assertIsInstance(arg, text_type)
+        self.assertIsInstance(arg, str)
 
 
-class TestGetOldestRequestTime(BuilderMixin, unittest.TestCase):
+class TestGetOldestRequestTime(TestReactorMixin, BuilderMixin,
+                               unittest.TestCase):
 
     @defer.inlineCallbacks
     def setUp(self):
+        self.setUpTestReactor()
         self.setUpBuilderMixin()
 
         # a collection of rows that would otherwise clutter up every test
@@ -532,10 +534,11 @@ class TestGetOldestRequestTime(BuilderMixin, unittest.TestCase):
         self.assertEqual(rqtime, None)
 
 
-class TestGetNewestCompleteTime(BuilderMixin, unittest.TestCase):
+class TestGetNewestCompleteTime(TestReactorMixin, BuilderMixin, unittest.TestCase):
 
     @defer.inlineCallbacks
     def setUp(self):
+        self.setUpTestReactor()
         self.setUpBuilderMixin()
 
         # a collection of rows that would otherwise clutter up every test
@@ -572,11 +575,12 @@ class TestGetNewestCompleteTime(BuilderMixin, unittest.TestCase):
         self.assertEqual(rqtime, None)
 
 
-class TestReconfig(BuilderMixin, unittest.TestCase):
+class TestReconfig(TestReactorMixin, BuilderMixin, unittest.TestCase):
 
     """Tests that a reconfig properly updates all attributes"""
 
     def setUp(self):
+        self.setUpTestReactor()
         self.setUpBuilderMixin()
 
     @defer.inlineCallbacks
