@@ -14,8 +14,6 @@
 # Copyright Buildbot Team Members
 
 # See "Type Validation" in master/docs/developer/tests.rst
-from future.utils import integer_types
-from future.utils import text_type
 
 import datetime
 import json
@@ -29,7 +27,7 @@ from buildbot.util import bytes2unicode
 validatorsByName = {}
 
 
-class Validator(object):
+class Validator:
 
     name = None
     hasArgs = False
@@ -59,7 +57,7 @@ class InstanceValidator(Validator):
 
 
 class IntValidator(InstanceValidator):
-    types = integer_types
+    types = (int,)
     name = 'integer'
 
 
@@ -70,7 +68,7 @@ class BooleanValidator(InstanceValidator):
 
 class StringValidator(InstanceValidator):
     # strings must be unicode
-    types = (text_type,)
+    types = (str,)
     name = 'string'
 
 
@@ -96,7 +94,7 @@ class DateTimeValidator(Validator):
 
 
 class IdentifierValidator(Validator):
-    types = (text_type,)
+    types = (str,)
     name = 'identifier'
     hasArgs = True
 
@@ -106,7 +104,7 @@ class IdentifierValidator(Validator):
         self.len = len
 
     def validate(self, name, object):
-        if not isinstance(object, text_type):
+        if not isinstance(object, str):
             yield "{} - {!r} - is not a unicode string".format(name, object)
         elif not self.ident_re.match(object):
             yield "{} - {!r} - is not an identifier".format(name, object)
@@ -119,7 +117,7 @@ class IdentifierValidator(Validator):
 # Miscellaneous
 
 
-class NoneOk(object):
+class NoneOk:
 
     def __init__(self, original):
         self.original = original
@@ -132,7 +130,7 @@ class NoneOk(object):
                 yield msg
 
 
-class Any(object):
+class Any:
 
     def validate(self, name, object):
         return
@@ -206,7 +204,7 @@ class StringListValidator(ListValidator):
     name = 'string-list'
 
     def __init__(self):
-        ListValidator.__init__(self, StringValidator())
+        super().__init__(StringValidator())
 
 
 class SourcedPropertiesValidator(Validator):
@@ -218,13 +216,13 @@ class SourcedPropertiesValidator(Validator):
             yield "{} is not sourced properties (not a dict)".format(name)
             return
         for k, v in object.items():
-            if not isinstance(k, text_type):
+            if not isinstance(k, str):
                 yield "{} property name {!r} is not unicode".format(name, k)
             if not isinstance(v, tuple) or len(v) != 2:
                 yield "{} property value for '{}' is not a 2-tuple".format(name, k)
                 return
             propval, propsrc = v
-            if not isinstance(propsrc, text_type):
+            if not isinstance(propsrc, str):
                 yield "{}[{}] source {!r} is not unicode".format(name, k, propsrc)
             try:
                 json.dumps(propval)
