@@ -7,7 +7,7 @@ from klein import Klein
 from twisted.internet import defer
 
 
-class Api(object):
+class Api:
     app = Klein()
     pizzaIngredients = {'margherita': ['tomato', 'ham', 'cheese'],
                         'regina': ['tomato', 'ham', 'cheese', 'mushrooms']}
